#ifndef _blackbox_menu_hh
#define _blackbox_menu_hh

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xresource.h>
#include <X11/cursorfont.h>
#include <X11/xpm.h>
#ifdef SHAPE
#include <X11/extensions/shape.h>
#endif

#include "llist.hh"

// forward declarations
class BlackboxMenu;
class BlackboxMenuItem;
class BlackboxSession;

// Note:  this is a base menu class... it is inherited and expanded upon for
// sessions, windows, and workspaces
class BlackboxMenu {
private:
  llist<BlackboxMenuItem> *menuitems;
  BlackboxSession *session;

  Bool moving, show_title;
  Display *display;
  GC titleGC;
  XContext itemContext;
  XFontStruct *menuFont, *titleFont;
  
  struct menu {
    char *label;
    int x, y, x_move, y_move;
    unsigned width, height,
      title_w, title_h,
      button_w, button_h, item_h;
    Pixel pframe, ptitle, pitem, ppushed;
    Pixmap item_pixmap, pushed_pixmap;
    Window frame, title;
  } menu;


protected:
  virtual void itemPressed(int, int) = 0;
  virtual void itemReleased(int, int) = 0;
  virtual void titlePressed(int) = 0;
  virtual void titleReleased(int) = 0;

  void showMenu(void);
  void hideMenu(void);
  void showTitle(void);
  void hideTitle(void);
  void moveMenu(int, int);
  void updateMenu(void);

  int insert(char *);
  int insert(char **);
  int insert(char *, int, char * = 0);
  int insert(char *, BlackboxMenu *);
  int remove(char *);
  int remove(int);
  BlackboxMenuItem *at(int);
  BlackboxSession *Session(void);

  void drawSubMenu(int);
  Window createItemWindow(void);


public:
  BlackboxMenu(BlackboxSession *);
  virtual ~BlackboxMenu(void);

  Window windowID(void);
  void setMenuLabel(char *);

  void buttonPressEvent(XButtonEvent *);
  void buttonReleaseEvent(XButtonEvent *);
  void motionNotifyEvent(XMotionEvent *);
  void exposeEvent(XExposeEvent *);
};


class BlackboxMenuItem {
public:
  BlackboxMenuItem(Window w, char **u)
    { window = w; ulabel = u; label = 0; exec = 0; sub_menu = 0;
      function = 0; }
  
  BlackboxMenuItem(Window w, char *l, int f)
    { window = w; ulabel = 0; label = l; exec = 0; sub_menu = 0;
      function = f; }
  
  BlackboxMenuItem(Window w, char *l, char *e)
    { window = w; ulabel = 0; label = l; exec = e; sub_menu = 0;
      function = 0; }

  BlackboxMenuItem(Window w, char *l)
    { window = w; ulabel = 0; label = l; exec = 0; sub_menu = 0;
      function = 0; }

  BlackboxMenuItem(Window w, char *l, BlackboxMenu *m)
    { window = w; ulabel = 0; label = l; sub_menu = m; exec = 0;
      function = 0; }

  ~BlackboxMenuItem(void)
    { }

  char *Exec(void) { return exec; }
  char *Label(void) { return label; }
  char **ULabel(void) { return ulabel; }
  int Function(void) { return function; }
  BlackboxMenu *Submenu(void) { return sub_menu; }

protected:


private:
  Window window;

  BlackboxMenu *sub_menu;

  char **ulabel, *label, *exec;
  int function;

  friend BlackboxMenu;
};


#endif
